@@ -154,6 +154,26 @@ y
 
 def get_efold_frequency(nu_min, nu_max, min_en=3., max_en=20., n_bins=32, unit='A',orbitfile=None,
                         actual_search=True):
+    """It finds the spin frequency using epoch folding
+
+    Args:
+        nu_min (float): the minimum search frequency
+        nu_max (float): the maximum search frequency
+        min_en (float, optional): the minimum energy of events to be used. Defaults to 3 keV.
+        max_en (float, optional):  the maximum energy of events to be used.. Defaults to 20 keV.
+        n_bins (int, optional): number of pulse bins to use. Defaults to 32.
+        unit (str, optional): the NuSTAR unit to use. Defaults to 'A'.
+        orbitfile (str, optional): The orbit file for correction. Defaults to None.
+        actual_search (bool, optional): If true it will run the epoch folding, 
+                                        if false it will read the output of aprevious search stored in the file ef_pipe_res.dat. 
+                                        Defaults to True.
+
+    Raises:
+        FileExistsError: FileExistsError in case the orbit file is given but not found
+
+    Returns:
+        float: the most probable spin frequency 
+    """                        
 
     if actual_search:
         with open('list_evt.txt', 'w') as ff:
@@ -809,7 +829,7 @@ def get_pulsed_fraction(e_min, e_max, pp, dpp, method_calc_rms='adaptive', outpu
         force_recompute (bool, optional): _description_. Defaults to False.
 
     Returns:
-        _type_: _description_
+       
         output files: 
                1. the fit result
                2. the fit results to be plotted in a file
@@ -1353,7 +1373,7 @@ def explore_fit_mcmc(model_fitted, pars=[], hm_steps=7000, hm_walkers=2000, hm_j
     return explore, emcee_plot
 
 
-def get_error_from_simul_rms(counts, counts_err, n_simul=100, n_harm=3):
+def  error_from_simul_rms(counts, counts_err, n_simul=100, n_harm=3):
 
     return get_error_from_simul(counts, counts_err, pulse_fraction_from_data_rms,
                                 n_simul=n_simul, n_harms=n_harm)
@@ -1762,8 +1782,26 @@ def write_orbit_file_from_gbm_page(url, file_name='orbit.dat',
             logger.info('%s %s' % (kk , orbit[kk]))
     return file_name
 
-def get_cross_correlation(pp, plot=False):
+def get_cross_correlation(pp, plot=False, n_to_fit = 3):
+    """makes the cross correlation and lags of a matrix by fitting a Gaussian plus constant to the peak of the 
+    correlation vector
 
+    Args:
+        pp (numpy array): the nergy or time-phase matrix
+        plot (bool, optional): if making debug plots. Defaults to False.
+        n_to_fit (int, optional): number of bins to use for the fit on each side of the peak. Defaults to 3.
+
+    Returns:
+        numpy arrays: lag, correlation
+    """
+    from lmfit.models import PolynomialModel, GaussianModel
+    poly_mod = PolynomialModel(prefix='poly_', degree=0)
+    gauss = GaussianModel(prefix='g'  + '_')
+
+    n_pulse=pp.shape[1]
+    if n_to_fit > n_pulse/3:
+        logger.warning('The pulse has size %s and you try to use % points' %(n_pulse, 2*n_to_fit+1))
+    
     correlation = []
     lag = []
     for i in range(pp.shape[0]):
@@ -1772,23 +1810,73 @@ def get_cross_correlation(pp, plot=False):
         y = np.sum(pp[0:i, :], 0) + np.sum(pp[i + 1:, :], 0)
         y = (y - np.mean(y)) / np.std(y)
         corr = np.correlate(np.tile(x, 2), y) / len(x)
-        correlation.append(np.max(corr))
-        lag.append(np.argmax(corr))
 
-    if plot:
-        import matplotlib.pyplot as plt
-        plt.figure()
-        plt.subplot(1, 2, 1)
-        plt.plot(corr)
-        plt.subplot(1, 2, 2)
-        plt.plot(x)
-        plt.plot(y, linestyle='--')
+        i_max = np.argmax(corr)
 
-    lag = np.array(lag)
-    lag[lag > (pp.shape[1]/2)] -= pp.shape[1]
+        if i_max - n_to_fit >= 0 and i_max + n_to_fit <= len(x) :
+            to_fit =  corr[i_max-n_to_fit:i_max+n_to_fit]
+        elif i_max - n_to_fit < 0:
+            to_fit = np.concatenate([corr[i_max-n_to_fit:], corr[0:i_max+n_to_fit]])
+        else:
+            to_fit = np.concatenate([corr[i_max-n_to_fit:], corr[1:i_max+n_to_fit-len(x)]])
+            
+        
+        x_to_fit = np.arange(i_max-n_to_fit,i_max+n_to_fit, dtype=float) 
+        
+        pars = poly_mod.guess(to_fit, x=x_to_fit, degree=0)
+        mod = poly_mod
+    
+        pars.update(gauss.make_params())
+        pars['g' +  '_center'].set(value=float(i_max), min=i_max-n_to_fit, max=i_max+n_to_fit)
+        pars['g' +  '_sigma'].set(value=1., min=0, max=n_to_fit)
+        pars['g' +  '_amplitude'].set(value=np.max(to_fit), min=0.0, max=2*np.max(to_fit))
+        mod = mod + gauss
+        
+        out = mod.fit(to_fit, pars, x=x_to_fit)
+
+        correlation.append(np.max(out.eval(x=np.linspace(x_to_fit[0], x_to_fit[-1], 100))))
+        lag.append(out.best_values['g' +  '_center'])
+
+        if plot:
+            import matplotlib.pyplot as plt
+            plt.figure()
+            plt.subplot(1, 2, 1)
+            plt.plot(corr)
+            plt.title('correlation')
+            plt.axvline(lag[-1])
+            plt.axhline(correlation[-1])
+            plt.subplot(1, 2, 2)
+            plt.plot(x, label='Pulse')
+            plt.plot(y, linestyle='--', label='Average')
+
+    lag = np.array(lag) / n_pulse
+    lag[lag > 0.5] -= 1.
     return lag, np.array(correlation)
 
-def get_cross_correlation_error(pp, dpp, n_simul=100, use_poisson=False):
+def get_cross_correlation_with_error(pp, dpp, ee_pulsed=[], dee_pulsed=[], n_simul=100, 
+                            use_poisson=False, n_to_fit=3, debug_plot=False, stem=None, title=''):
+    """Makes the cross correlation of each pulse in the matrix with the average of the rest of the matrix
+
+    Args:
+        pp (numpy array): energ-phase matrix
+        dpp (numpy array): energy-phase matrix error
+        ee_pulsed (list, optional): energy scale (only for plotting). Defaults to [].
+        dee_pulsed (list, optional): energy scale uncertainty (only for plotting). Defaults to [].
+        n_simul (int, optional): number of simulations to compute the error. Defaults to 100.
+        use_poisson (bool, optional): If using Poisson error. Defaults to False.
+        n_to_fit (int, optional): number of bins to fit the peak on each side of the maximum cross correlation. Defaults to 3.
+        debug_plot (bool, optional): if making debug plots. Defaults to False.
+        stem (_type_, optional): If not nome, it saves the plot with this prefix adding _lag_correlation.png. Defaults to None.
+        title (str, optional): Title of the plot. Defaults to ''.
+
+    Raises:
+        ValueError: Value Error if the energy scale does not match the matrix when plotting
+
+    Returns:
+        numpy arrays: lags, lag_errors, correlations, correlation_errors
+    """    
+
+    lags, correlations = get_cross_correlation(pp, n_to_fit=n_to_fit, plot=debug_plot)
 
     fake_lags = np.empty([n_simul, pp.shape[0]], dtype=float)
     fake_corrs = np.empty([n_simul, pp.shape[0]], dtype=float)
@@ -1799,9 +1887,25 @@ def get_cross_correlation_error(pp, dpp, n_simul=100, use_poisson=False):
         else:
             fake_pp = random.normal(pp, dpp)
 
-        fake_lags[i, :], fake_corrs[i, :] = get_cross_correlation(fake_pp)
+        fake_lags[i, :], fake_corrs[i, :] = get_cross_correlation(fake_pp, n_to_fit=n_to_fit)
+    
+    lag_errors = np.std(fake_lags, 0)
+    correlation_errors = np.std(fake_corrs, 0)
 
-    return np.std(fake_lags, 0), np.std(fake_corrs, 0)
+    if stem is not None:
+        if len(ee_pulsed) != len(lags):
+            raise ValueError('You should provide energy vetcors with size %d to plot' % len(lags))
+        
+        fig, (ax1, ax2) = plt.subplots(2, 1, figsize=(6, 8), sharex=True, gridspec_kw={'height_ratios': [1, 1],
+                                                                              'hspace': 0.0})
+        ax1.errorbar(ee_pulsed, correlations, xerr=dee_pulsed, yerr=correlation_errors, marker='.', linestyle='')
+        ax1.set_title(title)
+        ax1.set_ylabel('Correlation')
+        ax2.errorbar(ee_pulsed, lags, xerr=dee_pulsed, yerr=lag_errors, marker='x', linestyle='')
+        ax2.set_ylabel('Lag (phase units)')
+        fig.savefig(stem+'_lag_correlation.png' )
+
+    return lags, lag_errors, correlations, correlation_errors
 
 def get_target_coords_extern(input_name):
     from astroquery.simbad import Simbad
